package game;

import entities.Player;
import graphics.Credits;
import graphics.Level;
import graphics.LevelIndicator;
import graphics.Menu;
import java.awt.*;
import java.io.File;
import javax.sound.sampled.*;

public class Game implements Runnable {

  public static final String GAME_NAME = "Deez Nuts 1.0";

  public static final int FPS = 120;
  public static final int UPS = 250;

  public boolean music = false;
  private Clip clip;

  private Window window;
  private Panel panel;
  private Thread thread;

  public graphics.Menu menu;
  private final Credits credits;

  public Level level;

  public Game() {
    panel = new Panel(this);
    credits = new Credits(panel);

    window = new Window(panel);

    level = new Level(panel);
    menu = new Menu(panel, level, this);

    initMusic();
    startThread();
  }

  public void render(Graphics graphics) {
    switch (State.state) {
      case PLAY -> level.draw(graphics);
      case MENU -> menu.draw(graphics);
      case QUIT -> System.exit(0);
      case CREDITS -> credits.draw(graphics);
      default -> throw new IllegalStateException(
        "Unexpected value: " + State.state
      );
    }
  }

  private void update() {
    if (clip.isActive() && !music) {
      clip.stop();
    }

    if (!clip.isActive() && music) {
      clip.start();
    }

    switch (State.state) {
      case PLAY -> level.update();
      case MENU -> menu.update();
      case QUIT -> System.exit(0);
      case CREDITS -> credits.update();
      default -> throw new IllegalStateException(
        "Unexpected value: " + State.state
      );
    }
  }

  private void startThread() {
    thread = new Thread(this);
    thread.start();
  }

  private void initMusic() {
    try {
      File f = new File("assets/audio/background.wav");
      clip = AudioSystem.getClip();
      AudioInputStream audioIn = AudioSystem.getAudioInputStream(
        f.toURI().toURL()
      );

      clip.open(audioIn);
      clip.loop(Clip.LOOP_CONTINUOUSLY);
      clip.stop();
    } catch (Exception e) {
      System.out.println(e);
    }
  }

  @Override
  public void run() {
    double timePerFrame = 1000000000.0 / FPS; // 1s
    double timePerUpdate = 1000000000.0 / UPS; // 1s

    long previousTime = System.nanoTime();

    int frames = 0;
    int updates = 0;
    long lastCheck = System.currentTimeMillis();

    double deltaU = 0;
    double deltaF = 0;

    while (true) {
      long currentTime = System.nanoTime();

      deltaU += (currentTime - previousTime) / timePerUpdate;
      deltaF += (currentTime - previousTime) / timePerFrame;
      previousTime = currentTime;

      if (deltaU >= 1) {
        update();
        updates++;
        deltaU--;
      }

      if (deltaF >= 1) {
        panel.repaint();
        frames++;
        deltaF--;
      }

      if (System.currentTimeMillis() - lastCheck >= 1000) {
        lastCheck = System.currentTimeMillis();
        System.out.println("FPS: " + frames + " | UPS: " + updates);
        frames = 0;
        updates = 0;
      }
    }
  }
}
